/***************************************************************************************************
*                                       LibTopoART (usage example)                                 *
*                              created by Marko Tscherepanow, 23 July 2011                         *
***************************************************************************************************/

// compile with mcs /r:LibTopoART.dll example.cs

using System;
using System.IO;
using System.Text.RegularExpressions;
using LibTopoART;
using System.Globalization;

class Test_LibTopoART
{
	static void Main() {
		//dataset comprising two clusters, each containing 5 samples
		//cluster 1: samples 1 to 5
		//cluster 2: samples 6 to 10
		string Dataset="data/data.txt";		
		long SampleNumber=10;
		long InputDimension=2;
		StreamReader DatasetFile;
		decimal[][] InputArray=new decimal[SampleNumber][];
		long i;
		string Line;
		
		//load dataset
		DatasetFile=new StreamReader(File.Open(Dataset, FileMode.Open));
		for(i=0;i<SampleNumber;i++) {
			Line=DatasetFile.ReadLine();
			string[] numbers = Regex.Split(Line, @"\s+");
			InputArray[i]=new decimal[InputDimension]; 
			InputArray[i][0]=Decimal.Parse(numbers[1],NumberStyles.Float);
			InputArray[i][1]=Decimal.Parse(numbers[2],NumberStyles.Float);
			Console.WriteLine("Read input "+InputArray[i][0]+"; "+InputArray[i][1]);
		}
		DatasetFile.Close();
		
		//initialise a TopoART network using rho_a=0.7 and default values for the remaning parameters
		TopoART t=new TopoART(InputDimension,2,0.70m);		
		
		//train network
		for(i=0;i<SampleNumber;i++) {
			t.Learn(InputArray[i]);
		}
		
		//determine clusters
		t.ComputeClusterIDs();
		
		//save network in human readable form
		t.SaveText("TopoART.txt");
		
		//compute predictions for the randomly modified training data
		//(TopoART module 1 (TopoART b) creates a more detailed representation using 
		//more nodes than TopoART module 0 (TopoART a). But the number of clusters is equal.)
		Random rnd=new Random();
		F2_output[] output;
		decimal	x_deviation,y_deviation;
		for(i=0;i<SampleNumber;i++) {
			x_deviation=((decimal)rnd.Next(-1000,1000))/100000m;
			y_deviation=((decimal)rnd.Next(-1000,1000))/100000m;
			InputArray[i][0]+=x_deviation;
			InputArray[i][1]+=y_deviation;
			Console.WriteLine("Compute output for: "+InputArray[i][0]+"; "+InputArray[i][1]);
			output=t.GetBMOutput(InputArray[i]);
			Console.WriteLine("TopoART a: node {0}, cluster {1}",output[0].bm_node_ID,output[0].bm_cluster_ID);
			Console.WriteLine("TopoART b: node {0}, cluster {1}",output[1].bm_node_ID,output[1].bm_cluster_ID);
		}
	}
}